/**
 * SINGLETON
 */
package budrys.bca;

import android.app.Service;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;
import java.io.FileInputStream;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;
import budrys.chord.ChordDHT;
import budrys.chord.IfcCallback;
import budrys.security.Crypt;
import budrys.social.FriendsModel;
import budrys.social.ProfileModel;
import budrys.social.ProfileService;
import budrys.social.ProfileWallEntry;
import budrys.transport.socket.TransportClient;
import budrys.transport.socket.TransportServer;

public class BCAService extends Service {
	private NotificationManager nm;
    private Timer timer = new Timer();
    private int counter = 0, incrementby = 1;
    private static boolean isRunning = false;
    ArrayList<Messenger> mClients = new ArrayList<Messenger>(); // Keeps track of all current registered clients.
    int mValue = 0; // Holds last value set by a client.
    
    // commands 											// command issued by
    static final int MSG_REGISTER_CLIENT = 1;				// client, server
    static final int MSG_UNREGISTER_CLIENT = 2;				// client
    static final int MSG_SET_INT_VALUE = 3;					// 			deprecated
    static final int MSG_SET_STRING_VALUE = 4;				// 			deprecated
    static final int MSG_START_DHT_SERVICE = 5;				// client, server
    static final int MSG_REPORT_NEIGHBOURS = 6;				// client, server
    static final int MSG_DHT_ALREADY_RUNING = 7;			// server
    static final int MSG_START_DHT_SERVICE_STATUS = 8;		// server
    //social
//    static final int MSG_SOCIAL_INSERT_SAMPLE_PROFILES = 100;		// client, server
    static final int MSG_SOCIAL_PROFILE_PUBLISHED = 101;	// server
    // diagnostics
    static final int MSG_DIAG_DHT_LIST_PEERS = 1500;		// client, server
    static final int MSG_DIAG_DHT_LIST_PEERS_NEXT = 1501;	// server
    static final int MSG_DIAG_DHT_LIST_PEERS_FLOW = 1502;	// server
    // end commands
    
    final Messenger mMessenger = new Messenger(new IncomingHandler()); // Target we publish for clients to send messages to IncomingHandler.
    
    static public ChordDHT chord;
    static public ProfileService profileService;
    static public FriendsModel friends;

    @Override
    public IBinder onBind(Intent intent) {
        return mMessenger.getBinder();
    }
    class IncomingHandler extends Handler { // Handler of incoming messages from clients.
        @Override
        public void handleMessage(Message msg) {
        	Bundle args = msg.getData();
        	String arg1 = args.getString("arg1");
        	String arg2 = args.getString("arg2");
        	int arg3 = msg.arg1;
        	int arg4 = msg.arg2;
        	final Messenger replyTo = msg.replyTo;
            switch (msg.what) {
            case MSG_REGISTER_CLIENT:
                mClients.add(msg.replyTo);
                sendCommandToUI(msg.replyTo, MSG_REGISTER_CLIENT, null, null, 0, 0);
                if (chord != null){
	                String s = (chord.getSuccessor() == null) ? "waiting" : chord.getSuccessor().toString();
	        		String p = (chord.getPredecessorID() == null) ? "waiting" : chord.getPredecessorID().toString();
	        		sendCommandToUI(msg.replyTo, MSG_REPORT_NEIGHBOURS, s, p, 0, 0);
                }
                break;
            case MSG_UNREGISTER_CLIENT:
                mClients.remove(msg.replyTo);
                break;
            case MSG_SET_INT_VALUE:
                incrementby = msg.arg1;
                break;
            case MSG_START_DHT_SERVICE:
            	/// DHT
            	if (chord == null){
	                chord = new ChordDHT(arg1, (short)arg3, arg2, (short)arg4, new TransportServer(), new TransportClient());
	                // LOAD MY PROFILE FROM DISK
	                String profileFromDisk = AndroidFileFunctions.getFileValue(Define.BCA_PROFILE_FILENAME, getBaseContext());
	                ProfileModel pm;
	                if (profileFromDisk==null) {
	                	pm = new ProfileModel("Edit your", "Name", "Location", true);
	                	AndroidFileFunctions.setFileValue(Define.BCA_PROFILE_FILENAME, pm.exportAllToJson().toString(), getBaseContext());
	                } else {
	                	pm = new ProfileModel(profileFromDisk, true);
	                }
	                final ProfileModel finalPm = pm;
	                profileService = new ProfileService((short)(arg3+1), new TransportServer(), pm, friends, 
	                	new IfcCallback() { // onReceivePubMsg
						public void success(String command, String argument, Object callback) {
							AndroidFileFunctions.setFileValue(Define.BCA_PROFILE_FILENAME, finalPm.exportAllToJson().toString(), getBaseContext());
							ProfileWallEntry pwe = new ProfileWallEntry(argument);
							BCAService.this.showNotification(pwe.getName() + ": " + pwe.getMessage());
						}
						public void failure(Exception e, Object reason) {
							// not used
						}
					},new IfcCallback() { // onFileRequest
						public void success(String command, String argument, Object callback) {
							if (command.equals("avatar")){
								String path = Environment.getExternalStorageDirectory().toString();
								File f = new File(path, finalPm.getAvatarPath()); // or hardcoded avatar path
								if (!f.exists()){
									((IfcCallback) callback).failure(new Exception(), "FileNotFound");
								} else {
									FileInputStream fis;
									try {
										fis = new FileInputStream(f);
										int count;
										ArrayList<byte[]> bytesList = new ArrayList<byte[]>();
										byte[] justReceived = new byte[1024];
									    while ((count = fis.read(justReceived)) > 0) {
									    	bytesList.add(justReceived.clone());
									    }
									    byte[] wholeData = new byte[bytesList.size()*1024];
									    Iterator<byte[]> it = bytesList.iterator();
									    int destPos = 0;
									    while (it.hasNext()){
									    	byte[] c = it.next();
//									    	System.arraycopy(src, 	srcPos, dst, 		dstPos, 	length);
									    	System.arraycopy(c,		0, 		wholeData, 	destPos, 	c.length);
									    	destPos+=c.length;
									    }
									    ((IfcCallback) callback).success(null, null, wholeData);
									} catch (FileNotFoundException e) {
										// TODO Auto-generated catch block
										e.printStackTrace();
										((IfcCallback) callback).failure(e, null);
									} catch (IOException e) {
										// TODO Auto-generated catch block
										e.printStackTrace();
										((IfcCallback) callback).failure(e, null);
									}
								}
								
							}
						}
						public void failure(Exception e, Object reason) {
							// not used
						}
					});
	                // END LOAD MY PROFILE FROM DISK
	        		Log.i("CHORD", "My ID: "+chord.getIdHexString()+" my port:"+arg3+" remote port:"+arg4);
	        		chord.onNeighbourChange = new IfcCallback() {
						public void success(String successor, String predecessor, Object callback) {
							String s = (successor == null) ? "waiting" : successor.toString();
			        		String p = (predecessor == null) ? "waiting" : predecessor.toString();
			        		sendCommandToUI(replyTo, MSG_REPORT_NEIGHBOURS, s, p, 0, 0);
						}
						public void failure(Exception e, Object reason) {
						}
					};
					chord.startChordAsync(new IfcCallback() {
						public void success(String command, String argument, Object callback) {
							sendCommandToUI(replyTo, MSG_START_DHT_SERVICE_STATUS, "OK", null, 0, 0);
							// AUTO PUBLISH PROFILE
							if (Config.publishProfileOnStartup){
								new Runnable() {
									public void run() {
										while ((chord!=null)&&(chord.isRuning())){
											ProfileModel pm = BCAService.profileService.getProfileModel();
											String myDHTEntry = BCAService.chord.getMyIp() + ":" + BCAService.profileService.getMyPort().toString();
											if (pm.getPublicKey()!=null){
								        		BCAService.chord.dhtStore(Crypt.encodeKey(pm.getPublicKey()), myDHTEntry, new IfcCallback() {
								    				public void success(String command, String argument, Object callback) {			
								    					sendCommandToUI(replyTo, MSG_SOCIAL_PROFILE_PUBLISHED, "OK", null, 0, 0);
								    				}
								    				public void failure(Exception e, Object reason) {
//								    					sendCommandToUI(replyTo, MSG_SOCIAL_PROFILE_PUBLISHED, "ERROR", null, 0, 0);
								    				}
								    			});
								        	}
											if (Config.publishProfileNamesOnStartup){
									        	BCAService.chord.dhtStore(pm.getName(), myDHTEntry, new IfcCallback() {
													public void success(String command, String argument, Object callback) {					
													}
													public void failure(Exception e, Object reason) {
													}
												});
									        	BCAService.chord.dhtStore(pm.getSurname(), myDHTEntry, new IfcCallback() {
													public void success(String command, String argument, Object callback) {					
													}
													public void failure(Exception e, Object reason) {
													}
												});
									        	BCAService.chord.dhtStore(pm.getName()+" "+pm.getSurname(), myDHTEntry, new IfcCallback() {
													public void success(String command, String argument, Object callback) {					
													}
													public void failure(Exception e, Object reason) {
													}
												});
									        	BCAService.chord.dhtStore(pm.getSurname()+" "+pm.getName(), myDHTEntry, new IfcCallback() {
													public void success(String command, String argument, Object callback) {					
													}
													public void failure(Exception e, Object reason) {
													}
												});
									        	try {
													Thread.sleep(Config.republishTimePeriodMs);
												} catch (InterruptedException e1) {
													e1.printStackTrace();
												}
											}
										} // while
										
									} // run()
								}.run();
							}

						}
						public void failure(Exception e, Object reason) {
							sendCommandToUI(replyTo, MSG_START_DHT_SERVICE_STATUS, "FAIL", null, 0, 0);
						}
					});
	        		showNotification("CHORD ID: "+chord.getIdHexString());
	        		sendCommandToUI(replyTo, MSG_START_DHT_SERVICE, chord.getIdHexString(), null, 0, 0);
        		} else {
        			sendCommandToUI(replyTo, MSG_DHT_ALREADY_RUNING, chord.getIdHexString(), null, 0, 0);
        		}
                break;
            // SOCIAL
//            case MSG_SOCIAL_INSERT_SAMPLE_PROFILES:
//            	sendCommandToUI(msg.replyTo, MSG_SOCIAL_INSERT_SAMPLE_PROFILES, "in progress", null, 0, 0);
//            	chord.dhtStore("Jan Kowalski", "192.168.5.102:10105", new IfcCallback() {
//					public void success(String command, String argument, Object callback) {
//						sendCommandToUI(replyTo, MSG_SOCIAL_INSERT_SAMPLE_PROFILES, "ok", null, 0, 0);
//					}
//					public void failure(Exception e, Object reason) {
//						sendCommandToUI(replyTo, MSG_SOCIAL_INSERT_SAMPLE_PROFILES, "error", null, 0, 0);
//					}
//				});
//            	break;
            // DIAGNOSTICS
            case MSG_DIAG_DHT_LIST_PEERS:
            	sendCommandToUI(msg.replyTo, MSG_DIAG_DHT_LIST_PEERS, chord.getId().toString(), null, 0, 0);
            	chord.loopThroughPeers(new IfcCallback() {
            		// EACH
					public void success(String command, String argument, Object callback) {
						//Log.i("CHORD", command+": "+argument);
						sendCommandToUI(replyTo, MSG_DIAG_DHT_LIST_PEERS_NEXT, argument, null, 0, 0);
					}
					public void failure(Exception e, Object reason) {	// NOT USED
						sendCommandToUI(replyTo, MSG_DIAG_DHT_LIST_PEERS_FLOW, (String)reason, null, 0, 0);
					}
				}, new IfcCallback() {
            		// FINISH
					public void success(String command, String argument, Object callback) {
						sendCommandToUI(replyTo, MSG_DIAG_DHT_LIST_PEERS_FLOW, "FINISHED", null, 0, 0);
					}
					public void failure(Exception e, Object reason) {
						sendCommandToUI(replyTo, MSG_DIAG_DHT_LIST_PEERS_FLOW, (String)reason, null, 0, 0);
					}
				});
            	break;
            default:
                super.handleMessage(msg);
            }
        }
    }
    private void broadcastMessageToUI(int intvaluetosend) {
        for (int i=mClients.size()-1; i>=0; i--) {
            try {
                // Send data as an Integer
                mClients.get(i).send(Message.obtain(null, MSG_SET_INT_VALUE, intvaluetosend, 0));

            } catch (RemoteException e) {
                // The client is dead. Remove it from the list; we are going through the list from back to front so this is safe to do inside the loop.
                mClients.remove(i);
            }
        }
    }
    
//    private void broadcastCommandToUI(int command, String arg1, String arg2, int arg3, int arg4) {
//    	for (int i=mClients.size()-1; i>=0; i--) {
//            try {
//            	Bundle bundle = new Bundle();
//            	bundle.putString("arg1", arg1);	// no problem if null
//            	bundle.putString("arg2", arg2); // no problem if null               	
//                Message msg = Message.obtain(null, command, arg3, arg4);
//                msg.setData(bundle);
//                mClients.get(i).send(msg);
//
//            } catch (RemoteException e) {
//                // The client is dead. Remove it from the list; we are going through the list from back to front so this is safe to do inside the loop.
//                mClients.remove(i);
//            }
//        }
//    }
    
    private void sendCommandToUI(Messenger messenger, int command, String arg1, String arg2, int arg3, int arg4) {
        try {
        	Bundle bundle = new Bundle();
        	bundle.putString("arg1", arg1);	// no problem if null
        	bundle.putString("arg2", arg2); // no problem if null               	
            Message msg = Message.obtain(null, command, arg3, arg4);
            msg.setData(bundle);
            messenger.send(msg);
        } catch (RemoteException e) {
            // The client is dead.
        }
    }

    @Override
    public void onCreate() {
        super.onCreate();
        Log.i("MyService", "Service Started.");
        showNotification(getText(R.string.service_started));
        timer.scheduleAtFixedRate(new TimerTask(){ public void run() {onTimerTick();}}, 0, 1000L);
        isRunning = true;
        
        BCAService.createDirIfNotExists(Define.BCA_DIR);
        
        // load config
        {
	        String profileFromDisk = AndroidFileFunctions.getFileValue(Define.BCA_CONFIG_FILENAME, getBaseContext());
	        if (profileFromDisk==null) {
	        	AndroidFileFunctions.setFileValue(Define.BCA_CONFIG_FILENAME, Config.exportToJson(), getBaseContext());
	        } else {
	        	Config.importFromJson(profileFromDisk);
	        }
        }
        
       
	    // load friends
        {
	        String friendsFromDisk = AndroidFileFunctions.getFileValue(Define.BCA_FRIENDS_FILENAME, getBaseContext());
	        if (friendsFromDisk==null) {
	        	friends = new FriendsModel();
	        	AndroidFileFunctions.setFileValue(Define.BCA_FRIENDS_FILENAME, friends.exportAllToJson().toString(), getBaseContext());
	        } else {
	        	friends = new FriendsModel(friendsFromDisk);
	        }
        }
    }
    private void showNotification(CharSequence txt) {
        nm = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
        // In this sample, we'll use the same text for the ticker and the expanded notification
        CharSequence text = txt;
        // Set the icon, scrolling text and timestamp
        Notification notification = new Notification(R.drawable.ic_launcher, text, System.currentTimeMillis());
        // The PendingIntent to launch our activity if the user selects this notification
        PendingIntent contentIntent = PendingIntent.getActivity(this, 0, new Intent(this, BCAActivity.class), 0);
        // Set the info for the views that show in the notification panel.
        notification.setLatestEventInfo(this, getText(R.string.service_label), text, contentIntent);
        // Send the notification.
        // We use a layout id because it is a unique number.  We use it later to cancel.
        nm.notify(R.string.service_started, notification);
    }
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Log.i("MyService", "Received start id " + startId + ": " + intent);
        return START_STICKY; // run until explicitly stopped.
    }

    public static boolean isRunning()
    {
        return isRunning;
    }


    private void onTimerTick() {
        //Log.i("TimerTick", "Timer doing work." + counter);
        try {
            counter += incrementby;
            broadcastMessageToUI(counter);
            //Log.i("CHORD", chord.debugInfoJSON());
        } catch (Throwable t) { //you should always ultimately catch all exceptions in timer tasks.
            Log.e("TimerTick", "Timer Tick Failed on broadcast.", t);            
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // save config
        AndroidFileFunctions.setFileValue(Define.BCA_CONFIG_FILENAME, Config.exportToJson(), getBaseContext());
        
        if (timer != null) {timer.cancel();}
        counter=0;
        nm.cancel(R.string.service_started); // Cancel the persistent notification.
        Log.i("MyService", "Service Stopped.");
        isRunning = false;
        /// CHORD
        chord.stopChord();
        chord = null;
    }
    
    public static boolean createDirIfNotExists(String path) {
	    boolean ret = true;

	    File file = new File(Environment.getExternalStorageDirectory(), path);
	    if (!file.exists()) {
	        if (!file.mkdirs()) {
	        	System.out.println("Problem creating Image folder");
	            ret = false;
	        }
	    }
	    return ret;
	}
}
