package budrys.bca;

import java.util.ArrayList;
import java.util.Iterator;
import org.apache.commons.collections.map.MultiValueMap;
import budrys.chord.ID;
import budrys.chord.IfcCallback;
import android.content.ComponentName;
import android.content.Context;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

public class DebugActivity extends ServiceAbstractActivity {
	private Button btnListPeers, btnConnection, btnLocalStorage, btnInsertSample, btnFindSamples;
	private TextView textStatus;
	private ListView listPeers;
	
	private ArrayList<PeerItem> entries;
	
	private class IncomingHandler extends Handler {
        @Override
        public void handleMessage(Message msg) {
        	Bundle args = msg.getData();
        	String arg1 = args.getString("arg1");
//        	String arg2 = args.getString("arg2");
//        	int arg3 = msg.arg1;
//        	int arg4 = msg.arg2;
            switch (msg.what) {
            case BCAService.MSG_REGISTER_CLIENT:
            	textStatus.setText("2/2 Registered.");
            	break;
            case BCAService.MSG_DIAG_DHT_LIST_PEERS:
            	textStatus.setText("2/4 Acknowledge.");
            	clearPeersList();
            	ID myId = new ID(Long.valueOf(arg1));
            	addToPeersList(new PeerItem(myId.toString(), "Me - "+myId.getIP()+":"+myId.getPort().toString()) );
            	break;
            case BCAService.MSG_DIAG_DHT_LIST_PEERS_NEXT:
            	ID peerId = new ID(Long.valueOf(arg1));
        		addToPeersList(new PeerItem(peerId.toString(), "Peer - "+peerId.getIP()+":"+peerId.getPort().toString()) );
//            	try{
//            		
//            	} catch (NumberFormatException e){
//            		addToPeersList(new PeerItem(peerId.toString(), "Peer - "+peerId.getIP()+":"+peerId.getPort().toString()) );
//            	}
            	textStatus.setText("3/4 Completing ("+entries.size()+").");
            	break;
            case BCAService.MSG_DIAG_DHT_LIST_PEERS_FLOW:
            	textStatus.setText("4/4 Completed ("+entries.size()+").");
            	addToPeersList(new PeerItem(arg1, "INFO") );
            	break;
            default:
                super.handleMessage(msg);
            }
        }
    }
	
	public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.dht);
        
        // Required by ServiceActivity
        mConnection = new ServiceConnection() {
            public void onServiceConnected(ComponentName className, IBinder service) {
                mService = new Messenger(service);
                textStatus.setText("1/2 Attached.");
                try {
                    Message msg = Message.obtain(null, BCAService.MSG_REGISTER_CLIENT);
                    msg.replyTo = mMessenger;
                    mService.send(msg);
                } catch (RemoteException e) {
                    // In this case the service has crashed before we could even do anything with it
                }
            }

            public void onServiceDisconnected(ComponentName className) {
                // This is called when the connection with the service has been unexpectedly disconnected - process crashed.
                mService = null;
                textStatus.setText("Disconnected.");
            }
        };
        mMessenger = new Messenger(new IncomingHandler());
        
        
        
        btnListPeers = (Button)findViewById(R.id.btnListPeers);
        btnConnection = (Button)findViewById(R.id.btnConnection);
        btnLocalStorage = (Button)findViewById(R.id.btnLocalStorage);
        btnInsertSample = (Button)findViewById(R.id.btnInsertSample);
        btnFindSamples = (Button)findViewById(R.id.btnFindSamples);
        
        textStatus = (TextView)findViewById(R.id.textStatus);
        
        listPeers = (ListView)findViewById(R.id.listPeers);
        
        btnListPeers.setOnClickListener(btnListPeersListener);
        btnConnection.setOnClickListener(btnConnectionListener);
        btnLocalStorage.setOnClickListener(btnLocalStorageListener);
        btnInsertSample.setOnClickListener(btnInsertSampleListener);
        btnFindSamples.setOnClickListener(btnFindSamplesListener);
        // Required by ServiceActivity
        doBindService();
	}
	
	private void updatePeersList(){
		if (entries == null){
			entries = new ArrayList<PeerItem>();
		}
		PeerAdapter peersAdapter = new PeerAdapter(DebugActivity.this, R.layout.dht_peerslist,entries);
    	peersAdapter.setNotifyOnChange(true);
    	listPeers.setAdapter(peersAdapter);
    	listPeers.setTextFilterEnabled(true);
	}
	
	private void addToPeersList(PeerItem str){
		if (entries == null){
			entries = new ArrayList<PeerItem>();
		}
    	entries.add(0, str);
    	updatePeersList();
	}
	
	private void clearPeersList(){
		entries = new ArrayList<PeerItem>();
    	updatePeersList();
	}
	
	
	private OnClickListener btnListPeersListener = new OnClickListener() {
        public void onClick(View v){
        	textStatus.setText("1/4 Sending request.");
        	sendCommandToService(BCAService.MSG_DIAG_DHT_LIST_PEERS, null, null, 0, 0);
        }
    };
    
    private OnClickListener btnConnectionListener = new OnClickListener() {
        public void onClick(View v){
//        	btnConnection.setText("Going");
        	finish();
        }
    };
    
    private OnClickListener btnLocalStorageListener = new OnClickListener() {
        public void onClick(View v){
        	textStatus.setText("Checking storage");
        	clearPeersList();
        	MultiValueMap ha = BCAService.chord.getLocalStorage();
        	Iterator<String> en = ha.keySet().iterator();
        	if (!en.hasNext()){
        		addToPeersList(new PeerItem("empty", "empty"));
        	}
    		while (en.hasNext()){
    			String keyNow = en.next();
    			Iterator<String> vals = ha.getCollection(keyNow).iterator();
    			while(vals.hasNext()){
    				String key = keyNow;
            		String value = vals.next();
            		addToPeersList(new PeerItem(value, key));
    			}
    		}
        	textStatus.setText("Storage checked");
        }
    };
    
    private OnClickListener btnInsertSampleListener = new OnClickListener() {
        public void onClick(View v){
//        	DHTService.chord.dhtStore("Jan Kowalski", "192.168.5.102:11102", new IfcCallback() {
//				public void success(String command, String argument, Object callback) {					
//				}
//				public void failure(Exception e, Object reason) {
//				}
//			});
        	for (int i=10; i<20; ++i){
        		BCAService.chord.dhtStore("Ran"+i+"dom", "192.168.5.102:"+i, new IfcCallback() {
    				public void success(String command, String argument, Object callback) {					
    				}
    				public void failure(Exception e, Object reason) {
    				}
    			});
        	}
        }
    };
    
    private OnClickListener btnFindSamplesListener = new OnClickListener() {
        public void onClick(View v){
        	clearPeersList();
        	for (int i=10; i<20; ++i){
        		final int iBackup = i;
        		BCAService.chord.dhtGet("Ran"+i+"dom", new IfcCallback() {
					public void success(String command, String argument, Object callback) {
						final String key = argument;
		        		final String value = "Ran"+iBackup+"dom";
		        		runOnUiThread(new Runnable() {
		        			public void run() {
		        		    	 addToPeersList(new PeerItem(value, key));
		        		    	 textStatus.setText(entries.size()+"");
		        		    }
		        		});
		        		
					}
					
					public void failure(final Exception e, Object reason) {
						runOnUiThread(new Runnable() {
		        			public void run() {
		        				addToPeersList(new PeerItem("error", e.toString()));
		        			}
		        		});
					}
				});
        	}
        }
    };
	
    protected void onDestroy() {
        super.onDestroy();
        try {
            doUnbindService();
        } catch (Throwable t) {
            Log.e("BCAActivity", "Failed to unbind from the service", t);
        }
    }
    
    public class PeerItem {
    	private String top;
    	private String bottom;
    	
    	public PeerItem(String nTop, String nBottom){
    		this.setTop(nTop);
    		this.setBottom(nBottom);
    	}

		public String getTop() {
			return top;
		}

		private void setTop(String top) {
			this.top = top;
		}

		public String getBottom() {
			return bottom;
		}

		private void setBottom(String bottom) {
			this.bottom = bottom;
		}
    }
    
    private class PeerAdapter extends ArrayAdapter<PeerItem> {

        private ArrayList<PeerItem> items;

        public PeerAdapter(Context context, int textViewResourceId, ArrayList<PeerItem> items) {
                super(context, textViewResourceId, items);
                this.items = items;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
                View v = convertView;
                if (v == null) {
                    LayoutInflater vi = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                    v = vi.inflate(R.layout.dht_peerslist, null);
                }
                PeerItem o = items.get(position);
                if (o != null) {
                        TextView tt = (TextView) v.findViewById(R.id.toptext);
                        TextView bt = (TextView) v.findViewById(R.id.bottomtext);
                        if (tt != null) {
                              tt.setText(o.getTop());                            
                        }
                        if(bt != null){
                              bt.setText(o.getBottom());
                        }
                }
                return v;
        }
    }	// OrderAdapter
}
