package budrys.bca;

import java.io.File;
import java.io.FileOutputStream;

import budrys.social.ProfileModel;
import android.app.Activity;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.graphics.drawable.BitmapDrawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ImageView.ScaleType;
import android.widget.Toast;

public class EditProfileActivity extends Activity {
	private static final int SELECT_IMAGE = 0; 
	
	private Button btnEditProfileBack, btnEditProfileSave, btnSelectAvatar;
	private EditText editFirstName, editLastName, editLocation;
	private ImageView profileAvatarPreview;
	public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.editprofile);
        
        btnEditProfileBack = (Button)findViewById(R.id.btnEditProfileBack);
        btnEditProfileSave = (Button)findViewById(R.id.btnEditProfileSave);
        btnSelectAvatar = (Button)findViewById(R.id.btnSelectAvatar);
        
        editFirstName = (EditText)findViewById(R.id.editFirstName);
        editLastName = (EditText)findViewById(R.id.editLastName);
        editLocation = (EditText)findViewById(R.id.editLocation);
        
        profileAvatarPreview = (ImageView)findViewById(R.id.profileAvatarPreview);
        
        btnEditProfileBack.setOnClickListener(btnEditProfileBackListener);
        btnEditProfileSave.setOnClickListener(btnEditProfileSaveListener);
        btnSelectAvatar.setOnClickListener(btnSelectAvatarListener);
	}
	
	public void onResume(){
		super.onResume();
		ProfileModel pm = BCAService.profileService.getProfileModel();
		editFirstName.setText(pm.getName());
		editLastName.setText(pm.getSurname());
		editLocation.setText(pm.getAddress());
	}
	
	private OnClickListener btnEditProfileBackListener = new OnClickListener() {
        public void onClick(View v){
        	finish();
        }
    };
    
    private OnClickListener btnEditProfileSaveListener = new OnClickListener() {
        public void onClick(View v){
        	ProfileModel pm = BCAService.profileService.getProfileModel();
        	pm.setName(editFirstName.getText().toString());
        	pm.setSurname(editLastName.getText().toString());
        	pm.setAddress(editLocation.getText().toString());
        	pm.setRestricted(".");
        	
        	boolean saved = AndroidFileFunctions.setFileValue(Define.BCA_PROFILE_FILENAME, pm.exportAllToJson().toString(), getBaseContext());
        	if (saved){
        		easyToast("Profile saved successfully");        		
//        		easyToast(pm.exportAllToJson());
//        		Log.i("BCA.CRYPT", pm.exportAllToJson().toString());
        	} else {
        		easyToast("Error occurred while saving");
        	}
        	
        }
    };
    
    private OnClickListener btnSelectAvatarListener = new OnClickListener() {
        public void onClick(View v){
        	Intent gallery = new Intent(Intent.ACTION_PICK, android.provider.MediaStore.Images.Media.INTERNAL_CONTENT_URI);
            startActivityForResult(gallery, SELECT_IMAGE);
        }
    };
    
    protected void onActivityResult(int requestCode, int resultCode, Intent data) 
    {
        super.onActivityResult(requestCode, resultCode, data); 
        switch(requestCode) { 
        case SELECT_IMAGE:
            if(resultCode == RESULT_OK){  
                Uri selectedImage = data.getData();
                String[] filePathColumn = {MediaStore.Images.Media.DATA};

                Cursor cursor = getContentResolver().query(selectedImage, filePathColumn, null, null, null);
                cursor.moveToFirst();

                int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
                String filePath = cursor.getString(columnIndex);
                cursor.close();

                Bitmap selectedImageBitmap = BitmapFactory.decodeFile(filePath);
                
                int width = selectedImageBitmap.getWidth();
                int height = selectedImageBitmap.getHeight();
                int newWidth = 200;
                int newHeight = 200;
                float scaleWidth = ((float) newWidth) / width;
                float scaleHeight = ((float) newHeight) / height;

                Matrix matrix = new Matrix();
                matrix.postScale(scaleWidth, scaleHeight);
//                matrix.postRotate(45);

                // recreate the new Bitmap
                Bitmap resizedBitmap = Bitmap.createBitmap(selectedImageBitmap, 0, 0,
                                  width, height, matrix, true);
                
                try {
                	String path = Environment.getExternalStorageDirectory().toString();
                	File file = new File(path, Define.BCA_DIR+"avatar.png");
                	ProfileModel pm = BCAService.profileService.getProfileModel();
                	pm.setAvatarPath(file.getAbsolutePath());
                    FileOutputStream out = new FileOutputStream(file);
                    resizedBitmap.compress(Bitmap.CompressFormat.PNG, 90, out);
                    out.flush();
                    out.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }

                // make a Drawable from Bitmap to allow to set the BitMap
                // to the ImageView, ImageButton or what ever
                BitmapDrawable bmd = new BitmapDrawable(resizedBitmap);

                easyToast("bitmap: "+filePath);
//                profileAvatarPreview.setImageBitmap(selectedImageBitmap);
                profileAvatarPreview.setImageDrawable(bmd);
                profileAvatarPreview.setScaleType(ScaleType.CENTER);
                /* Now you have choosen image in Bitmap format in object "yourSelectedImage". You can use it in way you want! */
            }
        }

    };
    
    void easyToast(String text){
    	Toast toast = Toast.makeText(getApplicationContext(), text, Toast.LENGTH_SHORT);
    	toast.show();
    }
}
