package budrys.bca;

import java.util.ArrayList;
import java.util.Iterator;

import budrys.bca.SearchActivity.SearchItem;
import budrys.chord.IfcCallback;
import budrys.security.Crypt;
import budrys.social.FriendsModel;
import budrys.social.ProfileModel;
import budrys.social.RemoteProfile;
import budrys.transport.socket.TransportClient;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

public class FriendsActivity extends Activity {
	private Button btnFriendsBack;
	private ListView listWall;
	private ArrayList<FriendItem> entries;

	public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.friends);
        listWall = (ListView)findViewById(R.id.listWall);
        btnFriendsBack = (Button)findViewById(R.id.btnFriendsBack);
        btnFriendsBack.setOnClickListener(btnProfileBackListener);
        
        listWall.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				if (BCAService.isRunning()){
					final FriendItem fi = (FriendItem)listWall.getItemAtPosition(position);
					BCAService.chord.dhtGet(fi.getPublicKey(), new IfcCallback() {
						public void success(final String command, final String profileUrl, Object callback) {
	        				if (RemoteProfile.isValidUrl(profileUrl)){
	        					RemoteProfile rp = new RemoteProfile(profileUrl, new TransportClient());
		        				rp.getProfileData(Crypt.encodeKey(BCAService.profileService.getProfileModel().getPublicKey()), new IfcCallback() {
									public void success(String command2, final String argument2, Object callback2) {
										// FOUND IN DHT AND CONNECTED
										// UPDATE LOCAL CACHE
										final ProfileModel pm = new ProfileModel(argument2);
										BCAService.friends.updateFriend(fi.getPublicKey(), pm);
										AndroidFileFunctions.setFileValue(Define.BCA_FRIENDS_FILENAME, BCAService.friends.exportAllToJson().toString(), getBaseContext());
										// DISPLAY OBTAINED PROFILE DATA
										Intent i = new Intent(getApplicationContext(), ProfileActivity.class);
						        		i.putExtra("profileData", pm.exportAllToJson().toString());
						        		startActivity(i);
									}
									public void failure(Exception e, Object reason) {
										// ERROR ON GET PROFILE DATA
										// DISPLAY CACHED PROFILE DATA
										Intent i = new Intent(getApplicationContext(), ProfileActivity.class);
						        		i.putExtra("profileData", BCAService.friends.getFriend(fi.getPublicKey()).exportAllToJson().toString());
						        		i.putExtra("fromCache", true);
						        		startActivity(i);
									}
								});
	        				} else {
	        					// FOUND NOT VALID URL
	        					// DISPLAY CACHED PROFILE DATA
	        					Intent i = new Intent(getApplicationContext(), ProfileActivity.class);
				        		i.putExtra("profileData", BCAService.friends.getFriend(fi.getPublicKey()).exportAllToJson().toString());
				        		i.putExtra("fromCache", true);
				        		startActivity(i);
	        				}
						}
						public void failure(Exception e, final Object reason) {
							// NOT FOUNT IN DHT, MAY BE OFFLINE
							// DISPLAY CACHED PROFILE DATA
							Intent i = new Intent(getApplicationContext(), ProfileActivity.class);
			        		i.putExtra("profileData", BCAService.friends.getFriend(fi.getPublicKey()).exportAllToJson().toString());
			        		i.putExtra("fromCache", true);
			        		startActivity(i);
						}
					});
	        	} else {
	        		easyToast("Start the service first");
	        	}
			}
		});
    }
	
	public void onResume(){
		super.onResume();
		loadFromModel(BCAService.friends);
	}
	
	private OnClickListener btnProfileBackListener = new OnClickListener() {
        public void onClick(View v){
        	finish();
        }
    };
    
    void easyToast(String text){
    	Toast toast = Toast.makeText(getApplicationContext(), text, Toast.LENGTH_SHORT);
    	toast.show();
    }
    
    private void loadFromModel(FriendsModel fm){
    	clearFriendsList();
    	Iterator<ProfileModel> it = fm.enumFriends();
    	while (it.hasNext()){
    		ProfileModel pm = it.next();
    		addToFriendsList(new FriendItem(pm.getName() + " " + pm.getSurname(), pm.getAddress(), Crypt.encodeKey(pm.getPublicKey())), false);
    	}
    	updateFriendsList();
    }
    
	private void updateFriendsList(){
		if (entries == null){
			entries = new ArrayList<FriendItem>();
		}
		FriendsAdapter FriendsAdapter = new FriendsAdapter(FriendsActivity.this, R.layout.dht_peerslist,entries);
		FriendsAdapter.setNotifyOnChange(true);
    	listWall.setAdapter(FriendsAdapter);
    	listWall.setTextFilterEnabled(true);
	}
	
	private void addToFriendsList(FriendItem str){
		addToFriendsList(str, true);
	}
	
	private void addToFriendsList(FriendItem str, boolean update){
		if (entries == null){
			entries = new ArrayList<FriendItem>();
		}
    	entries.add(0, str);
    	if (update){
    		updateFriendsList();    		
    	}
	}
	
	private void clearFriendsList(){
		entries = new ArrayList<FriendItem>();
    	updateFriendsList();
	}
	
	public class FriendItem {
    	private String top;
    	private String bottom;
    	private String publicKey;
    	
    	public FriendItem(String nTop, String nBottom, String publicKey){
    		this.setTop(nTop);
    		this.setBottom(nBottom);
    		this.setPublicKey(publicKey);
    	}

		public String getPublicKey() {
			return publicKey;
		}
		
		private void setPublicKey(String publicKey) {
			this.publicKey = publicKey;
		}

		public String getTop() {
			return top;
		}

		private void setTop(String top) {
			this.top = top;
		}

		public String getBottom() {
			return bottom;
		}

		private void setBottom(String bottom) {
			this.bottom = bottom;
		}
    }
    
    private class FriendsAdapter extends ArrayAdapter<FriendItem> {

        private ArrayList<FriendItem> items;

        public FriendsAdapter(Context context, int textViewResourceId, ArrayList<FriendItem> items) {
                super(context, textViewResourceId, items);
                this.items = items;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
                View v = convertView;
                if (v == null) {
                    LayoutInflater vi = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                    v = vi.inflate(R.layout.friends_friendslist, null);
                }
                FriendItem o = items.get(position);
                if (o != null) {
                        TextView tt = (TextView) v.findViewById(R.id.toptext);
                        TextView bt = (TextView) v.findViewById(R.id.bottomtext);
                        TextView pt = (TextView) v.findViewById(R.id.publickeytext);
                        if (tt != null) {
                              tt.setText(o.getTop());                            
                        }
                        if(bt != null){
                              bt.setText(o.getBottom());
                        }
                        if(pt != null){
                            pt.setText(o.getPublicKey());
                        }
                }
                return v;
        }
    }
    
}
