package budrys.bca;

import java.util.ArrayList;
import budrys.chord.IfcCallback;
import budrys.social.ProfileModel;
import budrys.social.RemoteProfile;
import budrys.transport.socket.TransportClient;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public class SearchActivity extends ServiceAbstractActivity {
	private Button btnSocialSearch;
	private TextView textStatus;
	private ListView listSearchResults;
	private EditText editSearch;
	private ArrayList<SearchItem> entries;
	
	class IncomingHandlerSocial extends Handler {
        @Override
        public void handleMessage(Message msg) {
        	Bundle args = msg.getData();
        	String arg1 = args.getString("arg1");
        	String arg2 = args.getString("arg2");
        	int arg3 = msg.arg1;
        	int arg4 = msg.arg2;
            switch (msg.what) {
            case BCAService.MSG_REGISTER_CLIENT:
            	textStatus.setText("2/2 Registered.");
            	break;
//            case DHTService.MSG_SOCIAL_INSERT_SAMPLE_PROFILES:
//            	btnInsertSample.setText("Inserted: "+arg1);
//            	break;
            default:
                super.handleMessage(msg);
            }
        }
    }
	
	public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.social);
        
        // Required by ServiceActivity
        mConnection = new ServiceConnection() {
            public void onServiceConnected(ComponentName className, IBinder service) {
                mService = new Messenger(service);
                textStatus.setText("1/2 Attached.");
                try {
                    Message msg = Message.obtain(null, BCAService.MSG_REGISTER_CLIENT);
                    msg.replyTo = mMessenger;
                    mService.send(msg);
                } catch (RemoteException e) {
                    // In this case the service has crashed before we could even do anything with it
                }
            }

            public void onServiceDisconnected(ComponentName className) {
                // This is called when the connection with the service has been unexpectedly disconnected - process crashed.
                mService = null;
                textStatus.setText("Disconnected.");
            }
        };
        mMessenger = new Messenger(new IncomingHandlerSocial());
        
        btnSocialSearch = (Button)findViewById(R.id.btnSocialSearch);
        btnSocialSearch.setOnClickListener(btnSocialSearchListener);
        
        textStatus = (TextView)findViewById(R.id.textSocialStatus);
        listSearchResults = (ListView)findViewById(R.id.listSearchResults);
        editSearch = (EditText)findViewById(R.id.editSearch);
        
        listSearchResults.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				if (BCAService.isRunning()){
					SearchItem si = (SearchItem)listSearchResults.getItemAtPosition(position);
					if (si.getUrl().equals("NO")){
						easyToast("Cannot preview this entry");
					} else {
						// Profile preview
						Intent i = new Intent(getApplicationContext(), ProfileActivity.class);
		        		i.putExtra("profileUrl", si.getUrl());
		        		startActivity(i);
					}
	        		
	        	} else {
	        		easyToast("Start the service first");
	        	}
			}
		});
        
        //	Required by ServiceActivity
        doBindService();
	}
	
	public void onResume(){
		super.onResume();
		if (Config.lastSearchText != null){
			editSearch.setText(Config.lastSearchText);
		}
	}
	
	void easyToast(String text){
    	Toast toast = Toast.makeText(getApplicationContext(), text, Toast.LENGTH_SHORT);
    	toast.show();
    }
	
	private OnClickListener btnSocialSearchListener = new OnClickListener() {
        public void onClick(View v){
        	clearPeersList();
        	Config.lastSearchText = editSearch.getText().toString();
        	BCAService.chord.dhtGet(editSearch.getText().toString(), new IfcCallback() {
				public void success(final String command, final String profileUrl, Object callback) {
					runOnUiThread(new Runnable() {
	        			public void run() {
	        				if (RemoteProfile.isValidUrl(profileUrl)){
	        					RemoteProfile rp = new RemoteProfile(profileUrl, new TransportClient());
		        				rp.getProfileData(new IfcCallback() {
									public void success(String command2, final String argument2, Object callback2) {
										final ProfileModel pm = new ProfileModel(argument2);
										runOnUiThread(new Runnable() {
											public void run() {
												addToPeersList(new SearchItem(pm.getName() + " "+pm.getSurname(), pm.getAddress(), profileUrl));											
											}
										});
									}
									public void failure(Exception e, Object reason) {
										
									}
								});
	        				} else {
	        					addToPeersList(new SearchItem("NOPE", "Not found", "NO"));
	        				}
	        		    }
	        		});
										
				}
				public void failure(Exception e, final Object reason) {
					runOnUiThread(new Runnable() {
	        			public void run() {
	        				// TRY TO GET FROM ANOTHER NODE
	        				addToPeersList(new SearchItem("NOPE", (String)reason, "NO"));
	        		    }
	        		});
				}
			});
        	
        }
    };
	
    protected void onDestroy() {
        super.onDestroy();
        try {
            doUnbindService();
        } catch (Throwable t) {
            Log.e("BCAActivity", "Failed to unbind from the service", t);
        }
    }
    
    private void updatePeersList(){
		if (entries == null){
			entries = new ArrayList<SearchItem>();
		}
		SearchAdapter searchAdapter = new SearchAdapter(SearchActivity.this, R.layout.social_searchlist,entries);
		searchAdapter.setNotifyOnChange(true);
		listSearchResults.setAdapter(searchAdapter);
		listSearchResults.setTextFilterEnabled(true);
	}
	
	private void addToPeersList(SearchItem str){
		if (entries == null){
			entries = new ArrayList<SearchItem>();
		}
    	entries.add(0, str);
    	updatePeersList();
	}
	
	private void clearPeersList(){
		entries = new ArrayList<SearchItem>();
    	updatePeersList();
	}
	
	public class SearchItem {
    	private String top;
    	private String bottom;
    	private String url;
    	
    	public SearchItem(String nTop, String nBottom, String nUrl){
    		this.setTop(nTop);
    		this.setBottom(nBottom);
    		this.setUrl(nUrl);
    	}

		public String getUrl() {
			return url;
		}

		private void setUrl(String url) {
			this.url = url;
		}

		public String getTop() {
			return top;
		}

		private void setTop(String top) {
			this.top = top;
		}

		public String getBottom() {
			return bottom;
		}

		private void setBottom(String bottom) {
			this.bottom = bottom;
		}
    }
	
    private class SearchAdapter extends ArrayAdapter<SearchItem> {

        private ArrayList<SearchItem> items;

        public SearchAdapter(Context context, int textViewResourceId, ArrayList<SearchItem> items) {
                super(context, textViewResourceId, items);
                this.items = items;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
                View v = convertView;
                if (v == null) {
                    LayoutInflater vi = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                    v = vi.inflate(R.layout.social_searchlist, null);
                }
                SearchItem o = items.get(position);
                if (o != null) {
                        TextView tt = (TextView) v.findViewById(R.id.toptext);
                        TextView bt = (TextView) v.findViewById(R.id.bottomtext);
                        if (tt != null) {
                              tt.setText(o.getTop());                            
                        }
                        if(bt != null){
                              bt.setText(o.getBottom()+ " (" + o.getUrl() + ")");
                        }
                }
                return v;
        }
    }
}
