/**
 * ServiceActtivity
 * Contains methods to communicate with Service
 * To be extended by other classes
 */

package budrys.bca;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;
import android.widget.Toast;

public abstract class ServiceAbstractActivity extends Activity {
	protected Messenger mService = null;
	protected boolean mIsBound;
    /*static */protected Messenger mMessenger;				// to be set by child class
    protected ServiceConnection mConnection;	// to be set by child class
    
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }
    
    protected void CheckIfServiceIsRunning() {
        //If the service is running when the activity starts, we want to automatically bind to it.
        if (BCAService.isRunning()) {
            doBindService();
        }
    }
    
    protected void doBindService() {
        bindService(new Intent(this, BCAService.class), mConnection, Context.BIND_AUTO_CREATE);
        mIsBound = true;
        //textStatus.setText("Binding.");
    }
    protected void doUnbindService() {
        if (mIsBound) {
            // If we have received the service, and hence registered with it, then now is the time to unregister.
            if (mService != null) {
                try {
                    Message msg = Message.obtain(null, BCAService.MSG_UNREGISTER_CLIENT);
                    msg.replyTo = mMessenger;
                    mService.send(msg);
                } catch (RemoteException e) {
                    // There is nothing special we need to do if the service has crashed.
                }
            }
            // Detach our existing connection.
            unbindService(mConnection);
            mIsBound = false;
            //textStatus.setText("Unbinding.");
        } else {
        	Toast toast = Toast.makeText(getApplicationContext(), "Already not bound", Toast.LENGTH_SHORT);
        	toast.show();
        }
    }
    
    protected boolean sendMessageToService(int intvaluetosend) {
        if (mIsBound) {
            if (mService != null) {
                try {
                    Message msg = Message.obtain(null, BCAService.MSG_SET_INT_VALUE, intvaluetosend, 0);
                    msg.replyTo = mMessenger;
                    mService.send(msg);
                    return true;
                } catch (RemoteException e) {
                	return false;
                }
            } else {
            	return false;
            }
        } else {
        	return false;
        	//Toast toast = Toast.makeText(btnBind.getContext(), "Not bound to service", Toast.LENGTH_SHORT);
        	//toast.show();
        }
    }
    
    protected boolean sendCommandToService(int command, String arg1, String arg2, int arg3, int arg4) {
        if (mIsBound) {
            if (mService != null) {
                try {
                	Bundle bundle = new Bundle();
                	bundle.putString("arg1", arg1);	// no problem if null
                	bundle.putString("arg2", arg2); // no problem if null               	
                    Message msg = Message.obtain(null, command, arg3, arg4);
                    msg.setData(bundle);
                    msg.replyTo = mMessenger;
                    mService.send(msg);
                    return true;
                } catch (RemoteException e) {
                	return false;
                }
            } else {
            	return false;
            }
        } else {
        	return false;
        	//Toast toast = Toast.makeText(btnBind.getContext(), "Not bound to service", Toast.LENGTH_SHORT);
        	//toast.show();
        }
    }
    
    protected void onDestroy() {
        super.onDestroy();
        try {
            doUnbindService();
        } catch (Throwable t) {
            Log.e("BCAActivity", "Failed to unbind from the service", t);
        }
    }
    
}
