package budrys.chord;

import java.nio.ByteBuffer;
import java.nio.LongBuffer;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Locale;

import org.jsonb.*;
import org.apache.commons.collections.map.*;

import budrys.transport.socket.TransportClient;

public class ChordDHT extends Chord implements IfcDHT {
	
	// SOC: <hash(profile name), url_to_profile>
//	private Hashtable<String, String> storage;
	private MultiValueMap stor;

	public ChordDHT(String myIp, Short myPort, String peerIp, Short peerPort,
			IfcTransportServer ts, IfcTransportClient tc) {
		super(myIp, myPort, peerIp, peerPort, ts, tc);
//		storage = new Hashtable<String, String>();
		stor = new MultiValueMap();
		stor.put("test", "test");
	}
	
	public static ID idPlusOneThirdOfRange(ID source){
		long oneThirdOfLong = ((Long.MAX_VALUE / 3) - (Long.MIN_VALUE / 3)) + 1;
		return new ID(source.getID() + oneThirdOfLong);
	}
	
	public static ID idMinusOneThirdOfRange(ID source){
		long oneThirdOfLong = ((Long.MAX_VALUE / 3) - (Long.MIN_VALUE / 3)) + 1;
		return new ID(source.getID() - oneThirdOfLong);
	}
	
	// CHORD MAIN HASH FUNCTION
	public static ID hashChord(String key){
		key = key.toUpperCase(Locale.US);
		try {
			MessageDigest md = MessageDigest.getInstance("MD5");
			md.reset();
			md.update(key.getBytes());
			byte messageDigest[] = md.digest();
			ByteBuffer bb = ByteBuffer.wrap(messageDigest);
			LongBuffer lb = bb.asLongBuffer();
			return new ID(lb.get());
		} catch (NoSuchAlgorithmException nsae){
			return new ID((long)-1);
		}
	}
	
	public static ID hashChordOld(String key){
		int off = 0;
	    char val[] = key.toCharArray();
	    int len = key.length();
	    long h = 0;
        for (int i = 0; i < len; i++) {
            h = 31*h + val[off++];
        }
		Long result = (long)key.hashCode() << 32;
		return new ID(result);
	}
	
	// METHODS TO OPERATE ON LOCAL STORAGE
	/**
	 * LOCAL
	 * @param key
	 * @param value
	 * @return previous value or null when no previous value
	 */
	protected synchronized String storeLocal(String key, String value){
		key = key.toUpperCase(Locale.US);
		if (!this.stor.containsValue(key, value)){
			this.stor.put(key, value);
		}
		return null;
	}
	
	protected synchronized String storeLocal(String json){
		// expected input: {key: "key", value: ""}
		JSONObject decode;
		try {
			decode = new JSONObject(json.toString());
			String key = decode.getString("key");
			String value = decode.getString("value");
			return this.storeLocal(key, value);
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "{error-storage:\""+json+"\"}";
		}
	}
	
	/**
	 * LOCAL
	 * @param key
	 * @return value, or null
	 */
	protected synchronized Iterator<String> getLocal(String key){
		key = key.toUpperCase(Locale.US);
		Collection values = this.stor.getCollection(key);
		if (values!=null){
			return ((Iterator<String>) values.iterator());			
		} else {
			return null;
//			return new Iterator<String>() {
//				public boolean hasNext() {
//					return false;
//				}
//				public String next() {
//					return "";
//				}
//				public void remove() {
//				}
//			};
		}
	}
	
	/**
	 * LOCAL
	 * @param key
	 * @return previous value, or null
	 */
	
	protected synchronized String deleteLocal(String key){
		key = key.toUpperCase(Locale.US);
		this.stor.remove(key);
		return null;
	}
	
	protected synchronized String deleteLocal(String key, String value){
		key = key.toUpperCase(Locale.US);
		this.stor.remove(key, value);
		return null;
	}
	
	
			
	
	// TO BE CALLED FROM OUTSIDE THE CHORD
	
	/**
	 * Store data having 'key' on node id=successor('key')
	 */
	public synchronized ID dhtStore(final String key, final String value, final IfcCallback finalCallback) {
		dhtStore2(key.trim(), value, finalCallback);
		dhtStore3(key.trim(), value, finalCallback);
		return dhtStore1(key.trim(), value, finalCallback);
	}
	
	/**
	 * Store on original hash place
	 */
	private synchronized ID dhtStore1(final String key, final String value, final IfcCallback finalCallback) {
		PeerDHT peer = new PeerDHT(this.getId(), this.tcPattern);
		final ID keyHash = hashChord(key);
		peer.findSuccessor(keyHash.getID(), this.getId(), new IfcCallback() {
			
			public void success(String command, String argument, Object callback) {
				ID storagePeerID = new ID(Long.valueOf(argument));
//				Log.i("1Successor of key "+keyHash+" to store found: "+storagePeerID);
				PeerDHT storagePeer = new PeerDHT(storagePeerID.getID(), ChordDHT.this.tcPattern);
				storagePeer.dhtStore(key, value, finalCallback);
			}
			
			public void failure(Exception e, Object reason) {
				finalCallback.failure(e, reason);
			}
		});
		return hashChord(key);
	}
	
	/**
	 * Store on hash place + 1/3 of Long range
	 */
	private synchronized ID dhtStore2(final String key, final String value, final IfcCallback finalCallback) {
		PeerDHT peer = new PeerDHT(this.getId(), this.tcPattern);
		final ID keyHash = idPlusOneThirdOfRange(hashChord(key));
		peer.findSuccessor(keyHash.getID(), this.getId(), new IfcCallback() {
			
			public void success(String command, String argument, Object callback) {
				ID storagePeerID = new ID(Long.valueOf(argument));
//				Log.i("2Successor of key "+keyHash+" to store found: "+storagePeerID);
				PeerDHT storagePeer = new PeerDHT(storagePeerID.getID(), ChordDHT.this.tcPattern);
				storagePeer.dhtStore(key, value, finalCallback);
			}
			
			public void failure(Exception e, Object reason) {
				finalCallback.failure(e, reason);
			}
		});
		return hashChord(key);
	}
	
	/**
	 * Store on hash place - 1/3 of Long range
	 */
	private synchronized ID dhtStore3(final String key, final String value, final IfcCallback finalCallback) {
		PeerDHT peer = new PeerDHT(this.getId(), this.tcPattern);
		final ID keyHash = idMinusOneThirdOfRange(hashChord(key));
		peer.findSuccessor(keyHash.getID(), this.getId(), new IfcCallback() {
			
			public void success(String command, String argument, Object callback) {
				ID storagePeerID = new ID(Long.valueOf(argument));
//				Log.i("3Successor of key "+keyHash+" to store found: "+storagePeerID);
				PeerDHT storagePeer = new PeerDHT(storagePeerID.getID(), ChordDHT.this.tcPattern);
				storagePeer.dhtStore(key, value, finalCallback);
			}
			
			public void failure(Exception e, Object reason) {
				finalCallback.failure(e, reason);
			}
		});
		return hashChord(key);
	}

	public synchronized ID dhtGet(final String key, final IfcCallback finalCallback) {
		return dhtGet1(key, new IfcCallback() {
			public void success(String command, String argument, Object callback) {
				finalCallback.success(command, argument, callback);
			}
			public void failure(Exception e, Object reason) {
				dhtGet2(key, new IfcCallback() {
					public void success(String command, String argument, Object callback) {
						finalCallback.success(command, argument, callback);
					}
					public void failure(Exception e, Object reason) {
						dhtGet3(key, new IfcCallback() {
							public void success(String command, String argument, Object callback) {
								finalCallback.success(command, argument, callback);
							}
							public void failure(Exception e, Object reason) {
								finalCallback.failure(e, reason);
							}
						});
					}
				});
			}
		});
	}
	
	public synchronized ID dhtGet1(final String key, final IfcCallback finalCallback) {
		PeerDHT peer = new PeerDHT(this.getId(), this.tcPattern);
		ID keyHash = hashChord(key);
		peer.findSuccessor(keyHash.getID(), this.getId(), new IfcCallback() {
			public void success(String command, String argument, Object callback) {
				ID storagePeerID = new ID(Long.valueOf(argument));
//				Log.i("Successor of key to get found: "+storagePeerID);
				PeerDHT storagePeer = new PeerDHT(storagePeerID.getID(), ChordDHT.this.tcPattern);
				storagePeer.dhtGet(key, finalCallback);
			}
			
			public void failure(Exception e, Object reason) {
				finalCallback.failure(e, reason);
			}
		});
		return hashChord(key);
	}
	
	public synchronized ID dhtGet2(final String key, final IfcCallback finalCallback) {
		PeerDHT peer = new PeerDHT(this.getId(), this.tcPattern);
		ID keyHash = idPlusOneThirdOfRange(hashChord(key));
		peer.findSuccessor(keyHash.getID(), this.getId(), new IfcCallback() {
			public void success(String command, String argument, Object callback) {
				ID storagePeerID = new ID(Long.valueOf(argument));
//				Log.i("Successor of key to get found: "+storagePeerID);
				PeerDHT storagePeer = new PeerDHT(storagePeerID.getID(), ChordDHT.this.tcPattern);
				storagePeer.dhtGet(key, finalCallback);
			}
			public void failure(Exception e, Object reason) {
				finalCallback.failure(e, reason);
			}
		});
		return hashChord(key);
	}

	public synchronized ID dhtGet3(final String key, final IfcCallback finalCallback) {
		PeerDHT peer = new PeerDHT(this.getId(), this.tcPattern);
		ID keyHash = idMinusOneThirdOfRange(hashChord(key));
		peer.findSuccessor(keyHash.getID(), this.getId(), new IfcCallback() {
			public void success(String command, String argument, Object callback) {
				ID storagePeerID = new ID(Long.valueOf(argument));
//				Log.i("Successor of key to get found: "+storagePeerID);
				PeerDHT storagePeer = new PeerDHT(storagePeerID.getID(), ChordDHT.this.tcPattern);
				storagePeer.dhtGet(key, finalCallback);
			}
			public void failure(Exception e, Object reason) {
				finalCallback.failure(e, reason);
			}
		});
		return hashChord(key);
	}
	
	public synchronized ID dhtDelete(final String key, final IfcCallback finalCallback) {
		PeerDHT peer = new PeerDHT(this.getId(), this.tcPattern);
		ID keyHash = hashChord(key);
		peer.findSuccessor(keyHash.getID(), this.getId(), new IfcCallback() {
			
			public void success(String command, String argument, Object callback) {
				ID storagePeerID = new ID(Long.valueOf(argument));
				PeerDHT storagePeer = new PeerDHT(storagePeerID.getID(), ChordDHT.this.tcPattern);
				storagePeer.dhtGet(key, finalCallback);
			}
			
			public void failure(Exception e, Object reason) {
				finalCallback.failure(e, reason);
			}
		});
		return hashChord(key);
	}
	
	public MultiValueMap getLocalStorage(){
		return this.stor;
	}
	
	public void redistributeLocalStorage(ID caller) {
		Iterator<String> enu = this.stor.keySet().iterator();
		while (enu.hasNext()){
			String keyNow = enu.next();
			Iterator<String> vals = this.stor.getCollection(keyNow).iterator();
			while(vals.hasNext()){
				String val = vals.next();
				final String key = keyNow;
	    		final String value = val;
	    		dhtStore(key, value, new IfcCallback() {
					public void success(String command, String argument, Object callback) {
//						Log.i("redistributed ok, "+key+": "+value);
					}
					public void failure(Exception e, Object reason) {
						
					}
				});
			}
		}
	}
	
	public void onLeaveDHT(){
		// SHARE MY ENTRIES WITH NEIGHBOUR
		if (this.getSuccessor()==null){
			return;
		}
		final PeerDHT succ = new PeerDHT(this.getSuccessor().getID(), new TransportClient());
		Iterator<String> enu = this.stor.keySet().iterator();
		while (enu.hasNext()){
			String keyNow = enu.next();
			Iterator<String> vals = this.stor.getCollection(keyNow).iterator();
			while(vals.hasNext()){
				String val = vals.next();
				final String key = keyNow;
	    		final String value = val;
	    		succ.dhtStore(key, value, new IfcCallback() {
					public void success(String command, String argument, Object callback) {
					}
					public void failure(Exception e, Object reason) {
					}
				});
			}
		}
	}
	
	public synchronized String debugShowStoredEntries(){
		String result = "";
		
		Iterator<String> enu = this.stor.keySet().iterator();
		while (enu.hasNext()){
			String keyNow = enu.next();
			Iterator<String> vals = this.stor.getCollection(keyNow).iterator();
			while(vals.hasNext()){
				String val = vals.next();
				result += keyNow+":"+val+", ";
			}
		}
		return result;
	}

}
