/**
 * 
 * @author Micha Pawluczuk
 *
 * ID is outside of type Long
 * 
 * TODO teraz id=random+ip+port, pozamienia pewne ustalone bajty, aby hakerska manipulacja wasnego id bya znacznie utrudniona (wraliwa funkcja notify)
 * TODO wtedy dopisac funkcje do zwracania nielegalnych id (niemozliwych do uzyskania na podstawie id i zmapowac funkcj(od kolejnego numeru naturalnego w sumie z 0))
 * TODO wtedy zwracac w Chord.findSuccessor jako nielegalne wartosci
 * 
 */
package budrys.chord;

import java.util.Random;

public class ID {
	private Long idData;
	
	/**
	 * Generate a random id including IP address and PORT
	 * ID = random.concat(ip).concat(port)
	 * 
	 * @param ip
	 * @param port
	 */
	public ID(String ip, Short port){
		Random r = new Random();
		r.setSeed(System.currentTimeMillis());
		Integer rand = r.nextInt();
		idData = Math.abs(rand.longValue() << 48);
		idData += Long.valueOf(encodeIP(ip)) << 8*2;
		//idData = 0L;		// TODO do usuniecia na produkcji, powoduje ze id to numer portu
		idData += Long.valueOf(encodePort(port));
	}
	
	/**
	 * Converts a Long to a Chord ID
	 * 
	 * @param id
	 */
	public ID(Long id){
		this.idData = id;
	}
	
	public Long getID(){
		return idData;
	}
	
	public String getIP(){
		String result = "";
		Long mask = 0x00000000000000FFL;
		
		Long ipByte = ((idData >> 8*5) & mask);
		result += ipByte.toString() + ".";
		
		ipByte = ((idData >> 8*4) & mask);
		result += ipByte.toString() + ".";
		
		ipByte = ((idData >> 8*3) & mask);
		result += ipByte.toString() + ".";
		
		ipByte = ((idData >> 8*2) & mask);
		result += ipByte.toString();
		
		return result;
	}
	
	public Short getPort(){
		return (short)(idData & 0x000000000000FFFF);
	}
	
	public boolean isValid(){
		// TODO sprawdzic czy zawiera ip z dobrego range (bez klas loopback broadcast i LAN) oraz port
		return true;
	}
	
	public String toString(){
		//return idData.toBinaryString(idData);
		//return idData.toString();
		return Long.toHexString(idData);
	}
	
	/**
	 * 
	 * @param ip
	 * @return positive encoded value, or -1 if not a valid IPv4 address passed
	 */
	private int encodeIP(String ip){
		Integer result = 0;
		String[] tokens = ip.split("[.]");
		if (tokens.length != 4){
			return -1;
		}
		result += Integer.valueOf(tokens[0]) << 8*3;
		result += Integer.valueOf(tokens[1]) << 8*2;
		result += Integer.valueOf(tokens[2]) << 8*1;
		result += Integer.valueOf(tokens[3]) << 8*0;
		return result;
	}
	
	private int encodePort(Short port){
		return port.intValue();
	}
	
	public String debugInfo(){
		String result = "";
		result+="ID: " + this.toString()+"\n";
		result+="IP: " + this.getIP()+"\n";
		result+="Port: " + this.getPort()+"\n";
		result+="max: " + Long.MAX_VALUE+"\n";
		return result;
	}

}
