/**
 * 
 * @author Micha Pawluczuk
 *
 */
package budrys.transport.socket;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;

import budrys.chord.IfcCallback;
import budrys.chord.IfcTransportServer;

public class TransportServer implements IfcTransportServer {

	ServerSocket server;
	IfcCallback requestHandler;
	NewClientListener ncl;
	
	public Object cclone() {
		TransportServer result = new TransportServer();
		result.server = null;
		result.requestHandler = null;
		result.ncl = null;
		return result;
	}
	
	

	public boolean startService(Short port, IfcCallback requestHandler) {
		this.requestHandler = requestHandler;
		try {
			this.server = new ServerSocket(port);
		} catch (IOException e) {
			requestHandler.failure(e, null);
			e.printStackTrace();
			return false;
		}
		ncl = new NewClientListener(this.server, requestHandler);
		
		return true;
	}

	public boolean stopService() {
		
		// TODO terminate listener thread, and terminate serviceForClients
		
		try {
			if (this.server != null){
				this.server.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		this.server = null;
		ncl.callStop();
		return true;
	}

}


class NewClientListener extends Thread {
	private ServerSocket server;
	private IfcCallback requestHandler;
	
	public void callStop(){
		server = null;
	}
	
	public NewClientListener(ServerSocket server, IfcCallback requestHandler){
		this.server = server;
		this.requestHandler = requestHandler;
		start();
	}

	/**
	 * Thread routine
	 */
	public void run(){
		while ((server != null)&&(!server.isClosed())){
			try {
				Socket newClient = server.accept();
				new ServiceForClient(newClient, this.requestHandler);
			} catch (IOException e) {
//				System.out.println("ServiceForClient socket closed");
//				e.printStackTrace();
			}
		}
	}

}


class ServiceForClient extends Thread {
	private Socket client;
	private IfcCallback requestHandler;
	private boolean keepService = true;
	
	public ServiceForClient(Socket client, IfcCallback requestHandler){
		this.client = client;
		this.requestHandler = requestHandler;
		start();
	}
	
	/**
	 * Listening thread
	 */
	public void run(){
		
		while (this.keepService){
			ObjectInputStream in;
			try
			{
				in = new ObjectInputStream(this.client.getInputStream());
				
				try
				{
					TransportPacket receivedPacket = (TransportPacket) in.readObject();
					 // to handle client commands
					processRequest(receivedPacket);
				}
				catch (IOException e)
				{
					// exception here means that client got disconnected
					e.printStackTrace();
					keepService = false;
				}
				catch (ClassNotFoundException e)
				{
					e.printStackTrace();
					keepService = false;
				}
				finally{
					in.close();
				}
			}
			catch (EOFException eofe){
				//System.out.print("EOF!\n");
				// EOF = client.closed();
				keepService = false;
			}
			catch (IOException e){
				// exception here means that client got disconnected
				//System.out.print("DC???");
				//e.printStackTrace();
				keepService = false;	// this is only a service for this client for this time he has been connected
			}
			finally{
				try {
					client.close();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
			
			keepService = false;
			
		}
	} //run
	
	public void processRequest(TransportPacket requestPacket){
		this.requestHandler.success(requestPacket.command, requestPacket.argument, new IfcCallback() {
			public void success(String result1, String result2, Object callback) {
				// now we know what has client requested
				// A HACK to serve requests for binary: 
				//   callback != null means response is binary and binaryData = (byte[]) callback
				byte[] binaryData = (byte[])callback;
				if (binaryData==null){
					// text response, using TransportPacket
					ObjectOutputStream out;
					try{
						out = new ObjectOutputStream(client.getOutputStream());
						out.flush();
						
						try{
							out.writeObject(new TransportPacket(result1, (String)result2));
							// sent
						}
						catch (IOException e){
							client.close();
							keepService = false;
						} 
						finally {
							out.close();
						}
					}
					catch (IOException e){
						try{
							client.close();
							keepService = false;
						}
						catch (IOException e1){
							keepService = false;
						}
					} finally{
						//
					}
				} else {
					// binary response
					OutputStream os;
					try {
						os = new ObjectOutputStream(client.getOutputStream());
						os.write(binaryData);
						os.flush();
						os.close();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						try{
							client.close();
							keepService = false;
						}
						catch (IOException e1){
							keepService = false;
						}
					} finally {
						//
					}
					
				}
				
			}
			public void failure(Exception e, Object reason) {
				try {
					client.close();
					keepService = false;
				} catch (IOException e1) {
					keepService = false;
				}
			}
		});
	}
}
